#include "TArtCore.h"
#include "TArtStoreManager.h"
#include "TArtMap.h"
#include "TArtBigRIPSParameters.h"
#include "TArtPPACPara.h"
#include "TArtPlasticPara.h"
#include "TArtICPara.h"
#include "TArtFocalPlanePara.h"

#include <TROOT.h>
#include <TDOMParser.h>
#include <TXMLNode.h>

#include <stdlib.h>
#include <iostream>
#include <fstream>

//__________________________________________________________
TArtBigRIPSParameters::TArtBigRIPSParameters() : TNamed("BigRIPSParameters","BigRIPSParameters")
{
  TArtCore::Info(__FILE__,"Creating pid setup...");

  listOfPPACPara = new TList();
  listOfPlasticPara = new TList();
  listOfICPara = new TList();
  listOfFocalPlanePara = new TList();

  TArtStoreManager *sman = TArtStoreManager::Instance();
  sman->AddParameters(this);
}

//__________________________________________________________
TArtBigRIPSParameters::~TArtBigRIPSParameters() 
{
}

//__________________________________________________________
Bool_t TArtBigRIPSParameters::LoadParameter(char *xmlfile)
{
  TArtCore::Info(__FILE__,"Load parameter from %s", xmlfile);
  TDOMParser *domParser = new TDOMParser();
  domParser->SetValidate(false);
  Int_t parsecode = domParser->ParseFile(xmlfile);
  if (parsecode < 0) {
    cerr << domParser->GetParseCodeMessage(parsecode) << endl;
    return false;
  }
  TXMLNode * node = domParser->GetXMLDocument()->GetRootNode();
  ParseParaList(node->GetChildren());
  return true;
}

//__________________________________________________________
void TArtBigRIPSParameters::ParseParaList(TXMLNode *node) {
  for (; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "PPAC") == 0)
        listOfPPACPara->Add(ParsePPACPara(node->GetChildren()));
      if (strcmp(node->GetNodeName(), "Plastic") == 0)
        listOfPlasticPara->Add(ParsePlasticPara(node->GetChildren()));
      if (strcmp(node->GetNodeName(), "IC") == 0)
        listOfICPara->Add(ParseICPara(node->GetChildren()));
      if (strcmp(node->GetNodeName(), "FocalPlane") == 0)
        listOfFocalPlanePara->Add(ParseFocalPlanePara(node->GetChildren()));
    }
  }
  return;
}

//__________________________________________________________
TArtPPACPara *TArtBigRIPSParameters::ParsePPACPara(TXMLNode *node) {

  Int_t     id = -1;
  TString name;
  Int_t     fpl = 0;
  Double_t  a_ch2ns = 0, x1_ch2ns = 0, y1_ch2ns = 0, x2_ch2ns = 0, y2_ch2ns = 0;
  Double_t  xfactor = 0, yfactor = 0;
  Double_t  xoffset = 0, yoffset = 0;
  Double_t  xns_off = 0, yns_off = 0;
  Double_t  xpos_off = 0, ypos_off = 0;
  Double_t  xzpos = 0, yzpos = 0;
  Double_t  txsum_min = 0, txsum_max = 999999., tysum_min = 0, tysum_max = 999999.;

  Int_t Tx1_geo, Tx1_ch, Tx2_geo, Tx2_ch, Ty1_geo, Ty1_ch, Ty2_geo, Ty2_ch, Ta_geo, Ta_ch;
  Int_t Qx1_geo, Qx1_ch, Qx2_geo, Qx2_ch, Qy1_geo, Qy1_ch, Qy2_geo, Qy2_ch, Qa_geo, Qa_ch;

  for ( ; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "ID") == 0) 
	id = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "NAME") == 0) 
	name = node->GetText();
      if (strcmp(node->GetNodeName(), "FPL") == 0) 
	fpl = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "a_ch2ns") == 0)
	a_ch2ns = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "x1_ch2ns") == 0)
	x1_ch2ns = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "y1_ch2ns") == 0)
	y1_ch2ns = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "x2_ch2ns") == 0)
	x2_ch2ns = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "y2_ch2ns") == 0)
	y2_ch2ns = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "xfactor") == 0)
	xfactor = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "yfactor") == 0)
	yfactor = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "xoffset") == 0)
	xoffset = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "yoffset") == 0)
	yoffset = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "xns_off") == 0)
	xns_off = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "yns_off") == 0)
	yns_off = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "xpos_off") == 0)
	xpos_off = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "ypos_off") == 0)
	ypos_off = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "xzpos") == 0)
	xzpos = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "yzpos") == 0)
	yzpos = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "txsum_min") == 0)
	txsum_min = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "txsum_max") == 0)
	txsum_max = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "tysum_min") == 0)
	tysum_min = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "tysum_max") == 0)
	tysum_max = (Double_t)atof(node->GetText());


      if (strcmp(node->GetNodeName(), "Tx1_geo") == 0) 
	Tx1_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Tx1_ch") == 0) 
	Tx1_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Tx2_geo") == 0) 
	Tx2_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Tx2_ch") == 0) 
	Tx2_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Ty1_geo") == 0) 
	Ty1_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Ty1_ch") == 0) 
	Ty1_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Ty2_geo") == 0) 
	Ty2_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Ty2_ch") == 0) 
	Ty2_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Ta_geo") == 0) 
	Ta_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Ta_ch") == 0) 
	Ta_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qx1_geo") == 0) 
	Qx1_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qx1_ch") == 0) 
	Qx1_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qx2_geo") == 0) 
	Qx2_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qx2_ch") == 0) 
	Qx2_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qy1_geo") == 0) 
	Qy1_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qy1_ch") == 0) 
	Qy1_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qy2_geo") == 0) 
	Qy2_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qy2_ch") == 0) 
	Qy2_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qa_geo") == 0) 
	Qa_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "Qa_ch") == 0) 
	Qa_ch = atoi(node->GetText());

    }
  }

  TArtCore::Info(__FILE__,"Reading database for %s PPAC",name.Data());
  TArtPPACPara * para = new TArtPPACPara(id, name, fpl, a_ch2ns, x1_ch2ns, y1_ch2ns, x2_ch2ns, y2_ch2ns, xfactor, yfactor, xoffset, yoffset, xns_off, yns_off, xpos_off, ypos_off, xzpos, yzpos, txsum_min, txsum_max, tysum_min, tysum_max);
  para->SetMap(Tx1_geo, Tx1_ch, Tx2_geo, Tx2_ch, Ty1_geo, Ty1_ch, Ty2_geo, Ty2_ch, Ta_geo, Ta_ch, Qx1_geo, Qx1_ch, Qx2_geo, Qx2_ch, Qy1_geo, Qy1_ch, Qy2_geo, Qy2_ch, Qa_geo, Qa_ch);

  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetTX1Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetTX2Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetTY1Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetTY2Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetTAMap()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetQX1Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetQX2Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetQY1Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetQY2Map()), para));
  ppac_pmap.insert(std::pair<TArtRIDFMap, TArtPPACPara *>(*((TArtRIDFMap *)para->GetQAMap()), para));


 return para;

}

//__________________________________________________________
TArtPlasticPara *TArtBigRIPSParameters::ParsePlasticPara(TXMLNode *node) {

  Int_t     id = -1;
  TString name;
  Int_t     fpl = -1;
  Double_t  tcal_left = 0;
  Double_t  tcal_right = 0;
  Double_t  qped_left = 0;
  Double_t  qped_right = 0;
  Double_t  tslew_left_a = 0;
  Double_t  tslew_left_b = 0;
  Double_t  tslew_right_a = 0;
  Double_t  tslew_right_b = 0;

  Double_t  zpos = 0;
  Int_t tleft_geo, tleft_ch, tright_geo, tright_ch;
  Int_t qleft_geo, qleft_ch, qright_geo, qright_ch;

  for ( ; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "ID") == 0) 
	id = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "NAME") == 0) 
	name = node->GetText();
      if (strcmp(node->GetNodeName(), "FPL") == 0) 
	fpl = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "tcal_left") == 0)
	tcal_left = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "tcal_right") == 0)
	tcal_right = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "qped_left") == 0)
	qped_left = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "qped_right") == 0)
	qped_right = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(),"tslew_left_a")==0)
	tslew_left_a = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(),"tslew_left_b")==0)
	tslew_left_b = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(),"tslew_right_a")==0)
	tslew_right_a = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(),"tslew_right_b")==0)
	tslew_right_b = (Double_t)atof(node->GetText());

      if (strcmp(node->GetNodeName(), "zpos") == 0)
	zpos = (Double_t)atof(node->GetText());

      if (strcmp(node->GetNodeName(), "tleft_geo") == 0) 
	tleft_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "tleft_ch") == 0) 
	tleft_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "tright_geo") == 0) 
	tright_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "tright_ch") == 0) 
	tright_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "qleft_geo") == 0) 
	qleft_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "qleft_ch") == 0) 
	qleft_ch = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "qright_geo") == 0) 
	qright_geo = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "qright_ch") == 0) 
	qright_ch = atoi(node->GetText());
    }
  }
  
  TArtCore::Info(__FILE__,"Reading database for %s Plastic",name.Data());
  TArtPlasticPara * para = new TArtPlasticPara(id, name, fpl, tcal_left, tcal_right, qped_left, qped_right, tslew_left_a, tslew_left_b, tslew_right_a, tslew_right_b, zpos);
  para->SetMap(tleft_geo, tleft_ch, tright_geo, tright_ch, qleft_geo, qleft_ch, qright_geo, qright_ch);

  pla_pmap.insert(std::pair<TArtRIDFMap, TArtPlasticPara *>(*((TArtRIDFMap *)para->GetTLMap()), para));
  pla_pmap.insert(std::pair<TArtRIDFMap, TArtPlasticPara *>(*((TArtRIDFMap *)para->GetTRMap()), para));
  pla_pmap.insert(std::pair<TArtRIDFMap, TArtPlasticPara *>(*((TArtRIDFMap *)para->GetQLMap()), para));
  pla_pmap.insert(std::pair<TArtRIDFMap, TArtPlasticPara *>(*((TArtRIDFMap *)para->GetQRMap()), para));

  pla_name_pmap.insert(std::pair<TString, TArtPlasticPara *>(name, para));

  return para;

}

//__________________________________________________________
TArtICPara *TArtBigRIPSParameters::ParseICPara(TXMLNode *node) {

  Int_t     id = -1;
  TString name;
  Int_t     fpl = -1;
  Double_t  ch2mev_0 = 0, ch2mev_1 = 0;
  Double_t  zcoef_0 = 0, zcoef_1 = 0, zcoef_2 = 0;
  Double_t  ionpair = 0;
  Int_t geo[8], ch[8];

  for ( ; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "ID") == 0) 
	id = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "NAME") == 0) 
	name = node->GetText();
      if (strcmp(node->GetNodeName(), "FPL") == 0) 
	fpl = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch2mev_0") == 0)
	ch2mev_0 = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "ch2mev_1") == 0)
	ch2mev_1 = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "zcoef_0") == 0)
	zcoef_0 = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "zcoef_1") == 0)
	zcoef_1 = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "zcoef_2") == 0)
	zcoef_2 = (Double_t)atof(node->GetText());
      if (strcmp(node->GetNodeName(), "ionpair") == 0)
	ionpair = (Double_t)atof(node->GetText());

      if (strcmp(node->GetNodeName(), "geo0") == 0)
	geo[0] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo1") == 0)
	geo[1] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo2") == 0)
	geo[2] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo3") == 0)
	geo[3] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo4") == 0)
	geo[4] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo5") == 0)
	geo[5] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo6") == 0)
	geo[6] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "geo7") == 0)
	geo[7] = atoi(node->GetText());

      if (strcmp(node->GetNodeName(), "ch0") == 0)
	ch[0] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch1") == 0)
	ch[1] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch2") == 0)
	ch[2] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch3") == 0)
	ch[3] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch4") == 0)
	ch[4] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch5") == 0)
	ch[5] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch6") == 0)
	ch[6] = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "ch7") == 0)
	ch[7] = atoi(node->GetText());

    }
  }

  TArtCore::Info(__FILE__,"Reading database for %s IC",name.Data());
  TArtICPara * para = new TArtICPara(id, name, fpl, ch2mev_0, ch2mev_1, zcoef_0, zcoef_1, zcoef_2, ionpair);
  para->SetMap(geo, ch);

  for(int i=0;i<NUM_IC_CHANNEL;i++)
    ic_pmap.insert(std::pair<TArtRIDFMap, TArtICPara *>(*((TArtRIDFMap *)para->GetMap(i)), para));

  return para;

}

//__________________________________________________________
TArtFocalPlanePara *TArtBigRIPSParameters::ParseFocalPlanePara(TXMLNode *node) {

  Int_t     id = -1;
  Int_t     fpl = -1;
  Double_t  zpos = 0;

  for ( ; node; node = node->GetNextNode()) {
    if (node->GetNodeType() == TXMLNode::kXMLElementNode) { // Element Node
      if (strcmp(node->GetNodeName(), "ID") == 0) 
	id = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "FPL") == 0) 
	fpl = atoi(node->GetText());
      if (strcmp(node->GetNodeName(), "zpos") == 0)
	zpos = (Double_t)atof(node->GetText());
    }
  }

  TArtCore::Info(__FILE__,"Reading database for FocalPlane %d",fpl);
  TArtFocalPlanePara * para = new TArtFocalPlanePara(id,fpl,zpos);
  fpl_pmap.insert(std::pair<int, TArtFocalPlanePara *>(fpl, para));

  return para;

}

//__________________________________________________________
void TArtBigRIPSParameters::SetFocusPosOffset(Int_t fpl, Double_t offset) {

  TArtFocalPlanePara * fplpara = (TArtFocalPlanePara *)GetFocalPlanePara(fpl);
  if(NULL!=fplpara){
    fplpara->SetZoffset(offset);
    TArtCore::Info(__FILE__,"put z-offset on fpl %d: %f",fpl,(float)offset);
  }
  else{
    TArtCore::Info(__FILE__,"could not put z-offset on fpl %d: %f",fpl,(float)offset);
  }

  return;
}

//__________________________________________________________
TArtPPACPara * TArtBigRIPSParameters::GetPPACPara(TArtRIDFMap *rmap){
  std::map<TArtRIDFMap, TArtPPACPara *>::iterator itr = ppac_pmap.find(*rmap);
  if(itr != ppac_pmap.end()){
    return (TArtPPACPara*)itr->second;
  }
  return NULL;
}

//__________________________________________________________
TArtPlasticPara * TArtBigRIPSParameters::GetPlasticPara(TArtRIDFMap *rmap){
  std::map<TArtRIDFMap, TArtPlasticPara *>::iterator itr = pla_pmap.find(*rmap);
  if(itr != pla_pmap.end()){
    return (TArtPlasticPara*)itr->second;
  }
  return NULL;
}

//__________________________________________________________
TArtPlasticPara * TArtBigRIPSParameters::GetPlasticPara(TString *name){
  std::map<TString, TArtPlasticPara *>::iterator itr = pla_name_pmap.find(*name);
  if(itr != pla_name_pmap.end()){
    return (TArtPlasticPara*)itr->second;
  }
  return NULL;
}

//__________________________________________________________
TArtICPara * TArtBigRIPSParameters::GetICPara(TArtRIDFMap *rmap){
  std::map<TArtRIDFMap, TArtICPara *>::iterator itr = ic_pmap.find(*rmap);
  if(itr != ic_pmap.end()){
    return (TArtICPara*)itr->second;
  }
  return NULL;
}

//__________________________________________________________
TArtFocalPlanePara * TArtBigRIPSParameters::GetFocalPlanePara(int fpl){
  std::map<int, TArtFocalPlanePara *>::iterator itr = fpl_pmap.find(fpl);
  if(itr != fpl_pmap.end()){
    return (TArtFocalPlanePara*)itr->second;
  }
  return NULL;
}

